import {
  Button,
  Col,
  Collapse,
  Drawer,
  Form,
  Input,
  Row,
  Segmented,
  Select,
  Space,
  Table,
  Tabs,
  Typography,
} from "antd";
import { useTranslation } from "react-i18next";
import { useNavigate, useParams } from "react-router-dom";
import {
  AppStyle,
  Constants,
  myFetch,
  wsConnectionCustomEventName,
} from "../../utils";
import { useEffect, useState } from "react";
import { PlusOutlined } from "@ant-design/icons";
import { t } from "i18next";
import PageNotFound from "../PageNotFound";
import { useCrmContext } from "../../Contexts/CrmContext";
import MyPagination from "../../Components/MyPagination";

export default function Crm() {
  const { t } = useTranslation();
  const navigate = useNavigate();
  const crmContext = useCrmContext();
  const { paramType, paramDealPhase } = useParams();
  const [isDrawerOpen, setIsDrawerOpen] = useState(false);

  const title =
    paramType === Constants.CRM_TYPE.CUSTOMERS
      ? "crm.customers.pageTitle"
      : paramType === Constants.CRM_TYPE.DMC_PIPELINE
      ? "crm.dmcPipeline.pageTitle"
      : "crm.setterCloser.pageTitle";

  const segmentedOptions = t(
    paramType === Constants.CRM_TYPE.DMC_PIPELINE
      ? "crm.dmcPipeline.segmentedOptions"
      : "crm.setterCloser.segmentedOptions",
    {
      returnObjects: true,
    }
  );

  const [selectedSegmentedValue, setSelectedSegmentedValue] = useState([]);

  const getTableContent = () => {
    return [
      {
        title: t("crm.table.firstName"),
        dataIndex: "firstName",
        key: "firstName",
      },
      {
        title: t("crm.table.lastName"),
        dataIndex: "lastName",
        key: "lastName",
      },
      {
        title: t("crm.table.createdAt"),

        dataIndex: "createdAt",
        key: "createdAt",
      },
      {
        title: t("crm.table.telephone"),
        dataIndex: "telephone",
        key: "telephone",
      },
      {
        title: t("crm.table.email"),
        dataIndex: "email",
        key: "email",
      },
      {
        title: t("crm.table.lastContact"),
        dataIndex: "lastContact",
        key: "lastContact",
      },
      {
        title: t("crm.table.createdBy"),
        dataIndex: "createdBy",
        key: "createdBy",
      },
      {
        title: t("crm.table.notes"),
        dataIndex: "notes",
        key: "notes",
      },
    ];
  };

  const getTableItems = () => {
    let items = [];

    crmContext.customers.forEach((item) => {
      items.push({
        key: item.Id,
        id: item.Id,
        firstName: item.FirstName,
        lastName: item.LastName,
        createdAt: item.CreatedAt,
        telephone: item.Telephone,
        email: item.Email,
        lastContact: item.LastContact,
        createdBy: item.CreatedBy,
        notes: item.Notes,
      });
    });

    return items;
  };

  const onPaginationChange = (page) => {
    crmContext.setPaginationPage(page);
    crmContext.paginationPageRef.current = page;
  };

  useEffect(() => {
    if (paramType === Constants.CRM_TYPE.CUSTOMERS) return;

    setSelectedSegmentedValue(segmentedOptions[paramDealPhase - 1]);
  }, [paramType]);

  useEffect(() => {
    const customersRequest = () =>
      myFetch(
        `/crm/pipeline/${paramType}/${paramDealPhase}?page=${crmContext.paginationPage}`,
        "GET"
      ).then((data) => {
        if (data.Customers !== undefined) {
          crmContext.setCustomers(data.Customers);
        }

        if (data.TotalPages !== undefined) {
          crmContext.setTotalPages(data.TotalPages);
        }
      });

    customersRequest();

    const handleCustomersRequest = () => customersRequest();

    document.addEventListener(
      wsConnectionCustomEventName,
      handleCustomersRequest
    );

    return () =>
      document.removeEventListener(
        wsConnectionCustomEventName,
        handleCustomersRequest
      );
  }, [paramType, paramDealPhase]);

  if (paramDealPhase > segmentedOptions.length) {
    return <PageNotFound />;
  }

  return (
    <>
      <div
        style={{
          display: "flex",
          justifyContent: "space-between",
        }}
      >
        <Typography.Title level={4}>{t(title)}</Typography.Title>

        <Button icon={<PlusOutlined />} onClick={() => console.log("add")}>
          New
        </Button>
      </div>

      {paramType !== Constants.CRM_TYPE.CUSTOMERS && (
        <Segmented
          value={selectedSegmentedValue}
          onChange={(value) => {
            setSelectedSegmentedValue(value);
            navigate(
              `/crm/${paramType}/${segmentedOptions.indexOf(value) + 1}`,
              true
            );
          }}
          options={segmentedOptions.map((item) => {
            return {
              value: item,
              label: item,
            };
          })}
          style={{ marginBottom: AppStyle.app.margin }}
        />
      )}

      <Table
        scroll={{ x: "max-content" }}
        columns={getTableContent()}
        dataSource={getTableItems()}
        pagination={false}
        onRow={(record) => {
          return {
            onClick: () => {
              setIsDrawerOpen(true);
              crmContext.openDrawerCustomerId.current = record.id;
            },
          };
        }}
      />

      <MyPagination
        paginationPage={crmContext.paginationPage}
        setPaginationPage={(page) => onPaginationChange(page)}
        totalPages={crmContext.totalPages}
      />

      <CustomerDrawer
        isOpen={isDrawerOpen}
        onClose={() => setIsDrawerOpen(false)}
      />
    </>
  );
}

function CustomerDrawer({ isOpen, onClose }) {
  const { t } = useTranslation();
  const crmContext = useCrmContext();

  const tabItems = [
    {
      key: "0",
      label: t("crm.tabs.dealInfo"),
      children: <TabContentDealInfo />,
    },
    {
      key: "1",
      label: t("crm.tabs.activities"),
      children: "Tab 2 content",
    },
    {
      key: "2",
      label: t("crm.tabs.notes"),
      children: "Tab 3 content",
    },
  ];

  useEffect(() => {
    console.log("isOpen", isOpen);

    if (crmContext.openDrawerCustomerId.current === null) return;

    const customerRequest = () => {
      if (!isOpen) return;

      myFetch(
        `/crm/customer/view/${crmContext.openDrawerCustomerId.current}`,
        "GET"
      ).then((data) => {
        console.log("customer", data);
        crmContext.setCurrentDrawerCustomer(data);
        crmContext.currentDrawerCustomerRef.current = data;
      });
    };

    if (isOpen) {
      customerRequest();
    } else {
      // check if something has changed
      if (
        JSON.stringify(crmContext.currentDrawerCustomer) !==
        JSON.stringify(crmContext.currentDrawerCustomerRef.current)
      ) {
        console.log("something has changed");

        myFetch(
          `/crm/customer/update/${crmContext.openDrawerCustomerId.current}`,
          "POST",
          crmContext.currentDrawerCustomer
        ).then((data) => {
          console.log("update", data);
        });
      } else {
        console.log("nothing has changed");
      }

      crmContext.openDrawerCustomerId.current = null;
      crmContext.currentDrawerCustomer = null;
      crmContext.currentDrawerCustomerRef.current = null;
    }

    const handleCustomerRequest = () => customerRequest();

    document.addEventListener(
      wsConnectionCustomEventName,
      handleCustomerRequest
    );

    return () =>
      document.removeEventListener(
        wsConnectionCustomEventName,
        handleCustomerRequest
      );
  }, [isOpen]);

  return (
    <Drawer
      title={`${crmContext.currentDrawerCustomer?.FirstName} ${crmContext.currentDrawerCustomer?.LastName}`}
      placement="right"
      open={isOpen}
      onClose={onClose}
      width={720}
    >
      {console.log("drawer", crmContext.openDrawerCustomerId.current)}
      <Tabs defaultActiveKey="0" items={tabItems} centered />
    </Drawer>
  );
}

function Content({ children }) {
  return (
    <Form layout="vertical">
      <Row gutter={16}>
        <Col xs={24} sm={12}>
          {children[0]}
        </Col>
        <Col xs={24} sm={12}>
          {children[1]}
        </Col>
      </Row>
    </Form>
  );
}

function CollapseContainer({ children, label }) {
  return (
    <Collapse
      defaultActiveKey="1"
      items={[
        {
          key: "1",
          label: label,
          children: children,
        },
      ]}
    />
  );
}

function TabContentDealInfo() {
  const crmContext = useCrmContext();

  return (
    <Space direction="vertical" style={{ display: "flex" }}>
      <CollapseContainer
        label={t("crm.tabContent.dealInfo.collapseDealStatus.label")}
      >
        <Content>
          <Form.Item
            label={t("crm.tabContent.dealInfo.collapseDealStatus.pipeline")}
            initialValue={crmContext.currentDrawerCustomer?.Pipeline}
          >
            <Select
              value={crmContext.currentDrawerCustomer?.Pipeline}
              onChange={(value) => {
                //setSelectedPipeline(value);
                //setSelectedDealPhase(0);

                crmContext.setCurrentDrawerCustomer({
                  ...crmContext.currentDrawerCustomer,
                  Pipeline: value,
                  DealPhase: 0,
                });
              }}
              options={[
                {
                  value: 1,
                  label: t("crm.dmcPipeline.pageTitle"),
                },
                {
                  value: 2,
                  label: t("crm.setterCloser.pageTitle"),
                },
              ]}
            />
          </Form.Item>

          <Form.Item
            label={t("crm.tabContent.dealInfo.collapseDealStatus.dealPhase")}
            initialValue={crmContext.currentDrawerCustomer?.DealPhase}
          >
            <Select
              value={crmContext.currentDrawerCustomer?.DealPhase}
              onChange={(value) =>
                crmContext.setCurrentDrawerCustomer({
                  ...crmContext.currentDrawerCustomer,
                  DealPhase: value,
                })
              }
              options={t(
                crmContext.currentDrawerCustomer?.Pipeline === 1
                  ? "crm.dmcPipeline.segmentedOptions"
                  : "crm.setterCloser.segmentedOptions",
                { returnObjects: true }
              ).map((item, index) => {
                return {
                  value: index + 1,
                  label: item,
                };
              })}
            />
          </Form.Item>
        </Content>
      </CollapseContainer>

      <CollapseContainer
        label={t(
          t("crm.tabContent.dealInfo.collapseMasterDataOfContact.label")
        )}
      >
        <Content>
          <Form.Item
            label={t(
              "crm.tabContent.dealInfo.collapseMasterDataOfContact.firstName"
            )}
          >
            <Input
              value={crmContext.currentDrawerCustomer?.FirstName}
              onChange={(e) =>
                crmContext.setCurrentDrawerCustomer({
                  ...crmContext.currentDrawerCustomer,
                  FirstName: e.target.value,
                })
              }
            />
          </Form.Item>

          <Form.Item
            label={t(
              "crm.tabContent.dealInfo.collapseMasterDataOfContact.lastName"
            )}
          >
            <Input
              value={crmContext.currentDrawerCustomer?.LastName}
              onChange={(e) =>
                crmContext.setCurrentDrawerCustomer({
                  ...crmContext.currentDrawerCustomer,
                  LastName: e.target.value,
                })
              }
            />
          </Form.Item>
        </Content>

        <Content>
          <Form.Item
            label={t(
              "crm.tabContent.dealInfo.collapseMasterDataOfContact.telephone"
            )}
          >
            <Input
              value={crmContext.currentDrawerCustomer?.Telephone}
              onChange={(e) =>
                crmContext.setCurrentDrawerCustomer({
                  ...crmContext.currentDrawerCustomer,
                  Telephone: e.target.value,
                })
              }
            />
          </Form.Item>

          <Form.Item
            label={t(
              "crm.tabContent.dealInfo.collapseMasterDataOfContact.email"
            )}
          >
            <Input
              value={crmContext.currentDrawerCustomer?.Email}
              onChange={(e) =>
                crmContext.setCurrentDrawerCustomer({
                  ...crmContext.currentDrawerCustomer,
                  Email: e.target.value,
                })
              }
            />
          </Form.Item>
        </Content>

        <Content>
          <Form.Item
            label={t(
              "crm.tabContent.dealInfo.collapseMasterDataOfContact.company"
            )}
          >
            <Input />
          </Form.Item>

          <Form.Item
            label={t(
              "crm.tabContent.dealInfo.collapseMasterDataOfContact.zipCode"
            )}
          >
            <Input />
          </Form.Item>
        </Content>

        <Content>
          <Form.Item
            label={t(
              "crm.tabContent.dealInfo.collapseMasterDataOfContact.address"
            )}
          >
            <Input />
          </Form.Item>

          <Form.Item
            label={t(
              "crm.tabContent.dealInfo.collapseMasterDataOfContact.city"
            )}
          >
            <Input />
          </Form.Item>
        </Content>

        <Content>
          <Form.Item
            label={t(
              "crm.tabContent.dealInfo.collapseMasterDataOfContact.country"
            )}
          >
            <Input />
          </Form.Item>

          <Form.Item
            label={t(
              "crm.tabContent.dealInfo.collapseMasterDataOfContact.federalState"
            )}
          >
            <Input />
          </Form.Item>
        </Content>

        <Content>
          <Form.Item
            label={t(
              "crm.tabContent.dealInfo.collapseMasterDataOfContact.website"
            )}
          >
            <Input />
          </Form.Item>

          <Form.Item
            label={t(
              "crm.tabContent.dealInfo.collapseMasterDataOfContact.leadOrigin"
            )}
          >
            <Input />
          </Form.Item>
        </Content>
      </CollapseContainer>

      <CollapseContainer
        label={t(t("crm.tabContent.dealInfo.collapseSetterInfo.label"))}
      >
        <Content>
          <Form.Item
            label={t(
              "crm.tabContent.dealInfo.collapseSetterInfo.numberOfEmployees"
            )}
          >
            <Input />
          </Form.Item>

          <Form.Item
            label={t(
              "crm.tabContent.dealInfo.collapseSetterInfo.numberOfJobsSearchedFor"
            )}
          >
            <Input />
          </Form.Item>
        </Content>

        <Content>
          <Form.Item
            label={t("crm.tabContent.dealInfo.collapseSetterInfo.jobTitle")}
          >
            <Input />
          </Form.Item>

          <Form.Item
            label={t(
              "crm.tabContent.dealInfo.collapseSetterInfo.numberOfEmployeesRequired"
            )}
          >
            <Input />
          </Form.Item>
        </Content>

        <Content>
          <Form.Item
            label={t(
              "crm.tabContent.dealInfo.collapseSetterInfo.howLongHadHeBeenSearching"
            )}
          >
            <Input />
          </Form.Item>

          <Form.Item
            label={t("crm.tabContent.dealInfo.collapseSetterInfo.turnover")}
          >
            <Input />
          </Form.Item>
        </Content>
      </CollapseContainer>

      <CollapseContainer
        label={t(t("crm.tabContent.dealInfo.collapseDealProperties.label"))}
      >
        <Content>
          <Form.Item
            label={t(
              "crm.tabContent.dealInfo.collapseDealProperties.dateOfcompletion"
            )}
          >
            <Input />
          </Form.Item>

          <Form.Item
            label={t(
              "crm.tabContent.dealInfo.collapseDealProperties.orderVolume"
            )}
          >
            <Input />
          </Form.Item>
        </Content>

        <Content>
          <Form.Item
            label={t(
              "crm.tabContent.dealInfo.collapseDealProperties.numberOfInstallments"
            )}
          >
            <Input />
          </Form.Item>

          <Form.Item
            label={t(
              "crm.tabContent.dealInfo.collapseDealProperties.amountsOfTheInstallments"
            )}
          >
            <Input />
          </Form.Item>
        </Content>

        <Content>
          <Form.Item
            label={t(
              "crm.tabContent.dealInfo.collapseDealProperties.bookedPackages"
            )}
          >
            <Input />
          </Form.Item>

          <Form.Item
            label={t(
              "crm.tabContent.dealInfo.collapseDealProperties.assignedEmployee"
            )}
          >
            <Input />
          </Form.Item>
        </Content>
      </CollapseContainer>
    </Space>
  );
}
